<?php
/**
 * Ajax actions for the Full Screen Galleries plugin.
 */

/**
 * Attempts to generate additional gallery image data.
 *
 * Anyone can view front-end content, so nonces are not used/checked here.
 */
function full_screen_galleries_load_data() {

	// Figure out what image size is best to use for img src.
	// We're looking for something bigger than the image used in the content, but not unnecessarily large for the window size.
	$min_width = absint( $_POST['minWidth'] ); // Actually the larger of screen height or width to allow rotation.
	$best_size = '';
	$sizes = wp_get_registered_image_subsizes();

	foreach ( $sizes as $name => $size ) {
		if ( $size['crop'] ) {
			// We don't want to show a cropped image - looking for a big image.
			continue;
		} else {
			if ( $size['width'] < $min_width - 100 ) { // Include some tolerance in case there's a slightly smaller size but not a bigger size.
				continue;
			} elseif ( '' === $best_size || $size['width'] < $sizes[$best_size]['width'] ) {
				$best_size = $name;
			}
		}
	}

	if ( ! $best_size ) {
		// Didn't find a good size. See if the scaled-full size is close enough, or revert to large.
		if ( 2560 / 1.5 < $min_width ) { 
			// WP scaled "full" (original) image size is reasonably appropriate.
			$best_size = 'full';
		} else {
			// Large image size is probably best.
			$best_size = 'large';
		}
	}

	// Load image data.
	$data = array();
	$need_full = $_POST['needFullSizes'];
	$need_attachment = $_POST['needAttachmentInfo'];

	// Load data for images missing full-size information.
	if ( is_array( $need_full ) ) {
		foreach ( $need_full as $i => $image ) {
			$id = absint( $image['attachmentId'] );
			if ( $id ) {
				// Generate image full properties.
				$src = wp_get_attachment_image_src( $id, $best_size );
				$img = array(
					'fsgId' => absint( $image['fsgId'] ),
					'attachmentId' => $id,
					'src' => ( $src ) ? $src[0] : esc_url( $image['src'] ),
					// Link to the "full-size" image for additional (browser UI) zoom. Go bigger on larger screens.
					'url' => ( $min_width > 1200 ) ? wp_get_original_image_url( $id ) : wp_get_attachment_url( $id ),
				);
			} else {
				// Send back the inputs, but strip the invalid attachmentId.
				$img = array(
					'fsgId' => absint( $image['fsgId'] ),
					'attachmentId' => -1,
					'src' => esc_url( $image['src'] ),
					'url' => '',
				);
			}
			$data[] = $img;
		}
	}

	// Load data for images missing attachment information.
	if ( is_array( $need_attachment ) ) {
		foreach ( $need_attachment as $i => $image ) {
			$url = esc_url( $image['src'] );
			$id = absint( $image['attachmentId'] );
			if ( ! $id ) {
				// Try to find the attachment url from the post id.
				$id = attachment_url_to_postid( $url );
			}

			// attachment_url_to_postid only works for the primary "-scaled", full image size.
			// Attempt a few more things to get to an attachment id before giving up.
			if ( 0 === $id ) {
				// Append "-scaled" in case `src` is actually the original image URL.
				// @link https://core.trac.wordpress.org/ticket/51058
				$id = attachment_url_to_postid( substr( $url, 0, -4 ) . '-scaled' .  substr( $url, -4 ) ); // assumes a 3-character file extension
				if ( 0 === $id ) {
					// Attempt to strip the image size out of the url, which could also distort the original file name.
					// @link https://core.trac.wordpress.org/ticket/41816
					$url = preg_replace( '/-\d+[Xx]\d+\./', '.', $url );
					$id = attachment_url_to_postid( $url );
				}
			}

			if ( $id > 0 ) {
				// Generate image full properties.
				$src = wp_get_attachment_image_src( $id, $best_size );
				$img = array(
					'fsgId' => absint( $image['fsgId'] ),
					'attachmentId' => $id,
					'src' => ( $src ) ? $src[0] : esc_url( $image['src'] ),
					// Link to the "full-size" image for additional (browser UI) zoom. Go bigger on larger screens.
					// JS will keep the original URL if this fails due to an invalid ID.
					'url' => ( $min_width > 1200 ) ? wp_get_original_image_url( $id ) : wp_get_attachment_url( $id ),
				);
			} else {
				// Send back the inputs.
				$img = array(
					'fsgId' => absint( $image['fsgId'] ),
					'attachmentId' => -1,
					'src' => esc_url( $image['src'] ),
					'url' => '',
				);
			}
			$data[] = $img;
		}
	}

	if ( is_array( $data ) && ! empty( $data ) ) {
		wp_send_json_success( array( 'best_size' => $best_size, 'data' => $data ) );
	} else {
		wp_send_json_error( 'invalid_format' . var_dump( $need_full ) . var_dump( $need_attachment ) );
	}
}

add_action( 'wp_ajax_full-screen-gallery-details', 'full_screen_galleries_load_data' );
add_action( 'wp_ajax_nopriv_full-screen-gallery-details', 'full_screen_galleries_load_data' );
