/**
 * Full screen galleries script.
 *
 * Based on presentations.js and content-slideshow.js by Nick Halsey.
 *
 * Image data is generated from HTML markup initially. Additional data is requested via AJAX if needed, once the modal is opened.
 * Image HTML markup is generated from an HTML template and dynamically inserted into the DOM "just ahead of time".
 * Therefore, all data is implicitly lazy loaded to the extent feasible.
 */

var fsg = {};
( function( $, wp ) {
	fsg = {
		currentId : -1, // All IDs in JS are sequential gallery ids, not post ids.
		allData : {}, // Object containing data models for each post content (gallery) image.
		count: 0,
		needFullData : [], // Array of IDs that appear to be from block galleries and need full-size image data loaded.
		needAttachmentData : [], // Array of IDs that appear to be from classic galleries and need several image data items loaded.
		loadingData : false,
		width : 0,
		height : 0,
		template : {},
		container : {},
		imgContainer : {},
		contentContainer : {},
		mode : 'closed',
		t : 0,
		x1 : null,
		loaded : [], // Array of ids that have been loaded so far (not necessarily sequential).


		/*
		 * Initialize full-screen gallery scripts.
		 */
		init : function() {
			// Define container elements.
			fsg.container = $( '#fsg-container' );
			fsg.imgContainer = $( '#full-screen-gallery' );
			fsg.contentContainer = fsg.container.parent();

			// Generate the image gallery data model based on the existing HTML markup in the post content.
			var i = 0, imgId, imgs = fsg.contentContainer.find( 'img' );
			imgs.each( function( ii ) {
				if ( this.classList.contains( 'fsg-exclude' )
					 || this.parentElement.classList.contains( 'fsg-exclude' )
					 || this.parentElement.parentElement.classList.contains( 'fsg-exclude' ) ) {
					// Exclude this image from the full screen gallery (do nothing).
					// Third layer of hierarchy accounts for class on figure element for linked images.
				} else {
					if ( this.dataset.id ) {
						imgId = parseInt( this.dataset.id ), // WP attachment post ID

						// This image is likely in a block gallery.
						// All data should be availble from the markup.
						// However, the src may be undersized and the url is not to the actual full size.
						fsg.needFullData.push( i );
						
					} else if ( this.className && this.className.includes( 'wp-image' ) ) {
						// Fallback to get ID out of the className for image blocks that are not in galleries.
						imgId = parseInt( this.className.substr( 9 ) ); // Strip "wp-image-" from attribute value.

						// This image is likely in an image block. 
						// All data should be availble from the markup.
						// However, the src may be undersized and the url is not to the actual full size.
						fsg.needFullData.push( i );

					} else {
						if ( this.getAttribute( 'aria-describedby' ) ) {
							// Attachment ID is present here when there is an image caption.
							var desc = this.getAttribute( 'aria-describedby' );
							imgId = parseInt( desc.substr( 10 ) ); // Strip "gallery-X-" from attribute value. Will fail if there are more than 9 galleries in a post.
						} else {
							imgId = '';
						}

						// This image is likely in a classic gallery, or otherwise a random image tag in the content.
						// imgId, url, and attachmentLink are likely missing/blank/incorrect.
						// The src may also be undersized and should be reloaded.
						// We might be able to get the attachment ID from the ID element on the caption, but that would be fragile. See https://developer.wordpress.org/reference/functions/gallery_shortcode//
						fsg.needAttachmentData.push( i );
					}
					fsg.allData[i] = {
						id: i,
						imgId: imgId,
						src: this.src, // This is limited to whatever was in the_content. We'll look for bigger sizes via AJAX.
						srcset: this.srcset, // This is limited to whatever was in the_content.

						// Full-size image link (actually the scaled-down full size), or whatever the actual link is.
						url:  ( this.dataset.fullUrl ) ? this.dataset.fullUrl : $( this ).closest( 'a' ).attr( 'href' ),
						attachmentLink: this.dataset.link, // WP attachment page link
						caption: $( this ).closest( 'figure' ).find( 'figcaption' ).html(),
						alt: this.alt,
						orientation: ( this.width > this.height ) ? 'landscape' : 'portrait', // Determine orientation. This will fail for cropped images in the gallery.
					}

					// Label the content-image tag with the fsg ID.
					$( this ).attr( 'data-fsg-id', i );

					// Manually increment counter, because some images may be excluded.
					i = i + 1;
				}
			});

			// Abort early if there are fewer than three gallery images.
			fsg.count = _.size( fsg.allData );
			if ( 3 > fsg.count ) {
				return;
			}

			// Set up template.
			fsg.container.appendTo( document.body ); // Relocate the container to the end of the DOM to minimize CSS z-index layering and theme styling issues.
			fsg.width = window.innerWidth;
			fsg.height = window.innerHeight;
			fsg.template = wp.template( 'fsg-single-image-figure' );
			var counter = document.getElementById( 'fsg-progress-total' );
			counter.textContent = fsg.count;

			// Open the full-screen gallery to a specific image when the image is clicked in the content.
			fsg.contentContainer.find( 'img' ).on( 'click', function( e ) {
				if ( e.currentTarget.dataset.fsgId ) {
					e.preventDefault();
					fsg.showImage( e.currentTarget.dataset.fsgId, -1 );
				}
			});

			// Add event listeners for clicking on navigation buttons.
			fsg.container.find( '#fsg-close' ).on( 'click', function( e ) {
				fsg.closeModal();
			});
			fsg.container.find( '#fsg-next' ).on( 'click', function( e ) {
				fsg.nextImage();
			});
			fsg.container.find( '#fsg-previous' ).on( 'click', function( e ) {
				fsg.previousImage();
			});

			// Add event listeners for keyboard navigation.
			document.onkeydown = fsg.onkeydown;

			// Add event listeners to handle touch (swipe) navigation.
			// Note: this is a progressive enhancement not intended to be emulated with mouse interactions.
			var cont = document.getElementById( 'full-screen-gallery' );
			cont.addEventListener( 'touchstart', fsg.ontouchstart, true );
			cont.addEventListener( 'touchmove', fsg.ontouchend, true );

			// Add event listeners to, and show, launch button if added by a theme.
			var button = $( '#fsg-launch-button' );
			if ( 0 < button.length ) {
				button.show(); // CSS starts as display: none; in case there's no gallery.
				button.on( 'click', function() {
					fsg.launchModal();
				});
			}
		},

		/*
		 * Keyboard navigation handlers.
		 */
		onkeydown : function( e ) {
			if ( 37 === e.keyCode ) {
				// left arrow - navigate to previous image if modal is open
				if ( 'open' === fsg.mode ) {
					fsg.previousImage();
				}
			} else if ( 39 === e.keyCode ) {
				// right arrow - navigate to next image if modal is open
				if ( 'open' === fsg.mode ) {
					fsg.nextImage();
				}
			} else if ( 27 === e.keyCode ) {
				// esc key - close modal
				fsg.closeModal();
			} else if ( 9 === e.keyCode ) {
				// tab key - check for tabbing out of the modal
				if ( e.shiftKey ) {
					// Tabbing backward - close the modal if the current element is the image link.
					if ( 'fsg-close' === e.currentTarget.activeElement.id ) {
						fsg.closeModal();
					}
				} else {
					// Tabbing forward - close the modal if the current element is the previous button.
					if ( 'fsg-previous' === e.currentTarget.activeElement.id ) {
						fsg.closeModal();
					}
				}
			} else if ( 13 === e.keyCode ) {
				// enter key - open the model instead of triggering link if content image is selected
				if ( e.currentTarget.activeElement.children[0].dataset.fsgId ) { // Content images w/ FSG ID only
					e.preventDefault();
					fsg.showImage( e.currentTarget.activeElement.children[0].dataset.fsgId, -1 );
				}
			}
		},


		/*
		 * Touch event listeners to enable basic left/right swipe interactions
		 */
		ontouchstart : function( e ) {
			var initial = e.touches || evt.originalEvent.touches;
			fsg.x1 = initial[0].clientX;
		},

		ontouchend : function( e ) {
			if ( ! fsg.x1 ) {
				return;
			}

			e.preventDefault();

			var x2, end = e.touches || e.originalEvent.touches;
			x2 = end[0].clientX;

			if ( x2 < fsg.x1 - 16 ) {
				// Swipe to the right, show next image.
				fsg.nextImage();
			} else if ( x2 > fsg.x1 + 16 ) {
				// Swipe to the left, show previous image.
				fsg.previousImage();
			}

			// Reset
			fsg.x1 = null;
		},


		/*
		 * Open the modal to the first image.
		 */
		launchModal : function() {
			if ( 'open' !== fsg.mode ) {
				fsg.showImage( 0, -1 );
			}
		},


		/*
		 * Close the modal if it's open.
		 */
		closeModal : function() {
			if ( 'open' === fsg.mode ) {
				fsg.toggleMode();
			}
		},


		/*
		 * Open or close the full screen gallery browsing slideshow.
		 */
		toggleMode : function() {
			if ( 'closed' === fsg.mode ) {
				// Open the browsing modal.

				// Add body class.
				$( 'body' ).addClass( 'fsg-open' );

				// Prevent browser html element scroll.
				$( 'html' ).css( 'overflow-y', 'hidden' );

				// Show the container.
				fsg.container.show();

				// Focus is set in showImage().

				// Set mode to open.
				fsg.mode = 'open';
			} else {
				// Close the browsing modal.

				// Remove the body class.
				$( 'body' ).removeClass( 'fsg-open' );

				// Allow browser html element scroll.
				$( 'html' ).css( 'overflow-y', 'scroll' );

				// Hide the container.
				fsg.container.hide();

				// Hide the current image in the modal.
				var fig = document.getElementById( 'fsg-figure-' + fsg.currentId );
				fig.style.top = '100%';
				fig.style.opacity = '0';
				fig.getElementsByTagName('img')[0].tabIndex = -1; // 'a' is better if template includes link.

				// Set focus on the currentID image in the content.
				document.querySelectorAll("[data-fsg-id='" + fsg.currentId +"']")[0].parentElement.focus(); // @todo is there anything we can do when the img isn't linked?

				// Set mode to closed.
				fsg.mode = 'closed';
			}
		},


		/*
		 * Display an image, of any id in the sequence.
		 *
		 * Also pre-load adjacent sequential images.
		 *
		 * @param: imageID - numerical gallery id to display.
		 * @param: previous (optional) - numerical gallery id previously displayed, to hide.
		 */
		showImage : function( imageID, previous ) {
			imageID = parseInt( imageID );

			// Load image if not yet loaded
			if ( ! fsg.loaded.includes( imageID ) ) {
				fsg.loadImage( imageID );
			}

			// Verify previous image is specified.
			if ( ( undefined === previous || -1 === previous ) && fsg.currentID >= 0 ) {
				previous = fsg.currentID;
			}

			// Set current ID
			fsg.currentId = imageID;

			// Navigate modal to display selected image
			var fig = document.getElementById( 'fsg-figure-' + imageID );
			fig.style.top = '0';
			fig.style.opacity = '1'; // fades in via CSS transition
			var link = fig.getElementsByTagName('img')[0]; // 'a' is better if template includes an actual link.
			link.tabIndex = 0;
			link.focus();

			// Open modal if not yet open
			if ( 'closed' === fsg.mode ) {
				fsg.toggleMode();
			}

			// Set modal link to gallery image.
			fsg.container.find( '#fsg-open-full' ).attr( 'href', fsg.allData[imageID].url );

			// Update the progress counter.
			var number = document.getElementById( 'fsg-progress-number' ),
				slider = document.getElementById( 'fsg-progress-slider' ),
				percent = ( 1 + fsg.currentId ) / fsg.count * 100; // Start at 1/count, end at 100%.
			number.textContent = 1 + fsg.currentId;
			slider.style.width = percent + '%';

			// Pre-load next sequential image
			if ( fsg.currentId < fsg.count - 1 ) {
				fsg.loadImage( fsg.currentId + 1 );
			}

			// Pre-load previous sequential image w/ wraparound
			if ( fsg.currentId > 0 ) {
				fsg.loadImage( fsg.currentId - 1 );
			}

			// Hide the image that was previously shown, if exists.
			if ( undefined !== previous && previous >= 0 ) {
				var fig2 = document.getElementById( 'fsg-figure-' + previous );
				fig2.style.top = '100%';
				fig2.style.opacity = '0';
				fig2.getElementsByTagName('img')[0].tabIndex = -1; // 'a' is better if template includes link.
			}

			// Load supplementary data from Ajax if needed.
			if ( ( 1 < fsg.needFullData.length || 1 < fsg.needAttachmentData.length ) && ! fsg.loadingData ) {
				fsg.loadData();
			}
		},


		/*
		 * Load image markup. This does not make the image visible.
		 * This should trigger the browser to pre-load the image file.
		 */
		loadImage : function( imageID ) {
			imageID = parseInt( imageID );

			// Abort if image is already loaded.
			if ( fsg.loaded.includes( imageID ) ) {
				return;
			}

			// Generate the html markup for the image.
			var figure = fsg.template( fsg.allData[imageID] );

			// Add the html markup for the "next" image to the DOM.
			fsg.imgContainer.append( figure );

			// Add image to the list of loaded ids.
			fsg.loaded.push( imageID );
		},


		/*
		 * Display the next sequential image, with wraparound.
		 */
		nextImage : function() {
			var previous = parseInt( fsg.currentId, 10 );

			fsg.currentId = parseInt( fsg.currentId, 10 ) + 1;
			if ( fsg.currentId > fsg.count - 1 ) {
				// Warp around to the first image.
				fsg.currentId = 0;
			}

			// Show the next image.
			fsg.showImage( fsg.currentId, previous );
		},


		/*
		 * Display the previous sequential image, with wraparound.
		 */
		previousImage : function() {
			var previous = parseInt( fsg.currentId, 10 );

			fsg.currentId = parseInt( fsg.currentId, 10 ) - 1;
			if ( fsg.currentId < 0 ) {
				// Warp around to the last image.
				fsg.currentId = fsg.count - 1;
			}

			// Show the next image.
			fsg.showImage( fsg.currentId, previous );
		},


		/*
		 * Attempt to load additional gallery image data from WordPress via AJAX.
		 *
		 * This data is supplementary and treated as a progressive enhancement. Errors are therefore handled loosely.
		 */
		loadData : function() {
			// Do we need to load anything?
			if ( 1 > fsg.needFullData.length && 1 > fsg.needAttachmentData.length ) {
				return;
			}

			if ( fsg.loadingData ) {
				return; // Loading AJAX is already in progress.
			} else {
				fsg.loadingData = true;
			}

			var full = [], attachment = [];

			// Set up full-size request data based on fsg ids.
			fsg.needFullData.forEach( function( id ) {
				full.push({
					fsgId : id,
					attachmentId : fsg.allData[id].imgId,
					src : fsg.allData[id].src,
				});
				// Request needs to return the src calibrated to the target width, and the full image link url.
			});

			// Set up attachment data request data based on fsg ids.
			fsg.needAttachmentData.forEach( function( id ) {
				attachment.push({
					fsgId : id,
					attachmentId : fsg.allData[id].imgId, // Might've been succesfully parsed from aria-describedby.
					src : fsg.allData[id].src, // We need to try to find a better image size for this.
				});
				// Request needs to return the src calibrated to the target width, the full image link url,
				// the attachment id, and the attachment page link.
			});

			// Send request data to the server, and receive a response.
			wp.ajax.send( 'full-screen-gallery-details', {
				data: {
					minWidth: Math.max( fsg.width, fsg.height ),
					needFullSizes: full,
					needAttachmentInfo: attachment
				}
			} )
			.done( function( response ) {
				if ( response ) {
					// Validate returned data.
					if ( ! response.data || ! response.data.length ) {
						// Something went wrong. Abort and don't reset to try again.
						return;
					}

					// Regenerate gallery markup to incorporate added data.
					// This likely causes a content flash but should immediately restore the current view.

					// "Slip-sheet" returned data into the allData model.
					response.data.forEach( function( image ) {
						if ( undefined !== fsg.allData[image.fsgId] ) {
							fsg.allData[image.fsgId].imgId = parseInt( image.attachmentId );
							fsg.allData[image.fsgId].src = image.src;
							if ( '' !== image.url ) {
								// Only update the URL if it isn't blank.
								fsg.allData[image.fsgId].url = image.url;
							}
						}
					});

					var preloaded = fsg.loaded;
					fsg.loaded = [];

					// Remove all currently-loaded markup.
					fsg.imgContainer.html('');

					// Reload currently-loaded image markup.
					preloaded.forEach( function( id ) {
						fsg.loadImage( id );
					});

					// Show (re-display and focus) the current image.
					fsg.showImage( fsg.currentId, -1 );

					// Reset in case there's something else to load later (not likely).
					fsg.loadingData = false;
					fsg.needFullData = [];
					fsg.needAttachmentData = [];
				} else {
					console.log( 'Error processing full-screen gallery data request.' );
				}
			} )
			.fail( function( response ) {
				// If it's a known response code, show the corresponding error.
				if ( 'invalid_format' === response ) {
					console.log( 'invalid full screen gallery data request format' );
				} else {
					console.log( 'unkown error with full screen gallery data request' );
				}
				fsg.loadingData = false; // This will trigger loadData to potentially run/try again.
			} );
		},

	}

	$( document ).ready( function() { fsg.init(); } );

} )( jQuery, window.wp );